import es, path, zipfile, urllib, base64

info = es.AddonInfo()
info.version = '2.0rc3'
info.author = 'Ojii'
info.credits = 'Venjax for the ESAM SDK'

### CHANGELOG ###
#
# 2.0rc3:
#   -added clean() method to prevent any invalid characters in filename
#   -added builfileParser version 1, old buildfiles will be handled by buildfileParser version 0 (see documentation!)
#   -readded the uploadfile_creator (python) to the release
#   -added builder_version public variable
#   NOTE: Using parserversion 1 is highly recommended for any custom builds!
# 
# 2.0rc2:
#   -removed uploadfile_creator from release to get approved
#   -uploadfile_creator is now shipped via the forum thread
#
# 2.0rc1:
#   -first public 2.0 release
#   -added upload capability, added uploadfile
#   -added buildfile for custom builds
#   -added addinfo command for srcds uploadfile creation
#   -added uploadfile_creator for easier creation of uploadfiles


neededkeys = ['username','password','description','esversion','summary','install','loadname','version','releasenotes','fullname']
allkeys = ['username','password','description','esversion','summary','install','loadname','version','releasenotes','fullname','forumurl','publicvariable','featurelistbasename','videourl','ready','status']
invalid_characters = ['\\', '/', ':' ,'*' ,'?' ,'"' ,'<' ,'>', '|']
gp = path.path(str(es.ServerVar('eventscripts_gamedir')))

def load():
    es.regcmd('build','builder/build') # build basename
    es.regcmd('addinfo','builder/addinfo') # addinfo basename var value
    es.regcmd('upload','builder/upload') # upload basename
    es.ServerVar('builder_version', info.version).makepublic()
    print 'BUILDER: Loaded. You\'re using version %s' % info.version
    print 'BUILDER: Credits for the ESAM SDK go to Venjax, many thanks!'
    print 'BUILDER: Please report any bugs or problems on the EventScripts forums'
    print 'BUILDER: Please submit any feature requests you have to featurelist.org'
    
def clean(version):
    for character in invalid_characters:
        version = version.replace(character,'_')
    return version
    
def addinfo():
    if es.getargc() != 4:
        print 'BUILDER: Invalid syntax! Use: addinfo <basename> <variable> <value>'
    name = es.getargv(1)
    var = es.getargv(2)
    value = es.getargv(3)
    ap = path.path(es.getAddonPath(es.getargv(1)))
    if not ap.isdir():
        print 'BUILDER: Couldn\'t find addon folder!'
        return
    if not var in allkeys:
        print 'BUILDER: Invalid variable!'
        print 'BUILDER: Possible variables:'
        print 'BUILDER: %s' % ','.join(allkeys)
    uf = ap.joinpath('%s.upload' % name)
    L = uf.lines(retain=False)
    done = False
    x = 0
    for line in L:
        if line.startswith(var):
            del L[x]
            L.insert(x,'%s %s' % (var,value))
            done = True
            break
        x += 1
    if not done:
        L.append('%s %s' % (var,value))
    tobedeleted = []
    for line in L:
        ok = False
        for key in allkeys:
            if line.startswith(key):
                ok = True
        if not ok:
            tobedeleted.append(line)
    for line in tobedeleted:
        L.remove(line)
    uf.write_lines(L)
    print 'BUILDER: Added info: %s %s' % (var,value)

def upload():
    if es.getargc() != 2:
        print 'BUILDER: Invalid syntax! Use: upload <basename>'
        return
    ap = path.path(es.getAddonPath(es.getargv(1)))
    if not ap.isdir():
        print 'BUILDER: Couldn\'t find addon folder!'
        return
    name = es.getargv(1)
    uf = ap.joinpath('%s.upload' % name)
    if not uf.isfile():
        print 'BUILDER: Uploadfile not found!'
        print 'BUILDER: Use addinfo <basename> <var> <value> to add info or add manuall'
        return
    L = uf.lines(retain=False)
    ud = {}
    for line in L:
        key,val = line.split(' ',1)
        ud[key] = val
    for key in neededkeys:
        if not ud.has_key(key):
            print 'BUILDER: Not enough information in upload file!'
            print 'BUILDER: Use addinfo <basename> <var> <value> to add info or add manuall'
            return
    zf = ap.joinpath('%s_%s.zip' % (name,clean(ud['version'])))
    if not zf.isfile():
        print 'BUILDER: ZIP Archive not found!'
        print 'BUILDER: Make sure you built the addon before you try to upload!'
        print 'BUILDER: Use build <basename> to build it'
        return
    ### UPLOADER ###
    ESAMURL = 'http://addons.eventscripts.com/sdk/upload'
    print 'BUILDER: Starting upload...'
    b64c = base64.b64encode(zf.bytes())
    print 'BUILDER: Depending on your filesize (in this case: %s Bytes) this might take a while. Grab a beer (or a tea if you\'re a minor)!' % len(b64c)
    raw_params = [
        ('username',ud['username']),
        ('password',ud['password']),
        ('basename',name),
        ('filename',str(zf.basename())),
        ('b64_file_content',b64c),
        ('filesize',zf.getsize()),
        ('file_extension',str(zf.ext)[1:]),
        ('addon_desc',ud['description'].replace('\\n','''
''')),
        ('es_version',ud['esversion']),
        ('addon_summary',ud['summary'][:255]),
        ('install_instructions',ud['install'].replace('\\n','''
''')),
        ('es_load_args',ud['loadname']),
        ('version',ud['version']),
        ('version_notes',ud['releasenotes'].replace('\\n','''
''')),
        ('addon_name',ud['fullname']),
        ('forum_url',ud['forumurl'] if ud.has_key('forumurl') else ''),
        ('public_variable',ud['publicvariable'] if ud.has_key('publicvariable') else ''),
        ('featurelist_url',ud['featurelistbasename'] if ud.has_key('featurlistbasename') else ''),
        ('video_url',ud['videourl'] if ud.has_key('videourl') else ''),
        ('ready4approval',ud['ready'] if ud.has_key('ready') else '1'),
        ('status',ud['status'] if ud.has_key('status') else 'Normal'),
    ]
    params = urllib.urlencode(raw_params)
    upload = urllib.urlopen(ESAMURL,params)
    response = upload.read()
    print 'BUILDER: Upload finished!'
    print 'BUILDER: %s' % response

 ### EXPERIMENTAL! ###
 # Does it work?
def extractDownloadable(line):
    if 'es.stringable' in line: # 2.x
        start = line.find('es.stringtable')
        end = line.find(')',start)
        tline = line[stp+14:end] # line between es.stringtable( and )
        table,filepath = tline.split(',')
        table = table.strip().strip('"').strip("'").strip()
        if table == 'downloadables':
            filepath = filepath.strip().strip('"').strip("'").strip()
            download = gp.joinpath(filepath[1:]) if filepath[0] in ('\\','/') else gp.joinpath(filepath)
            if download.isfile():
                return download
    elif 'es_stringtable' in line: # 1.x
        command, table, filepath = line.split(' ',2)
        if table == 'downloadables':
            filepath = filepath.strip().strip('"').strip("'").strip()
            download = gp.joinpath(filepath[1:]) if filepath[0] in ('\\','/') else gp.joinpath(filepath)
            if download.isfile():
                return download
    elif 'downloadable' in line: # 1.x corelib
        command, filepath = line.split(' ',1)
        filepath = filepath.strip().strip('"').strip("'").strip()
        download = gp.joinpath(filepath[1:]) if filepath[0] in ('\\','/') else gp.joinpath(filepath)
        if download.isfile():
            return download
    return False
    
def buildfileParser1(lines, bf):
    print 'BUILDER: Parsing buildfile using parser version 1'
    exclude = []
    strict = False
    buildfileFiles = []
    extensions = ['py','txt','ini','cfg','res']
    for line in lines: # scan for strict mode and extensions
        if line == '__strict__':
            strict = True
            print 'BUILDER: strict mode enabled'
        elif line.startswith('__extensions__'):
            extensions = line[14:].split(',')
    for line in lines: # do a new loop because of the extensions
        if not line.startswith('#'): # comments...
            if line[0:2] == '++':
                line = line[2:].strip()
                f = gp.joinpath(line[1:]) if line[0] in ('\\','/') else gp.joinpath(line)
                if f.isfile():
                    buildfileFiles.append(f)
                elif f.isdir():
                    for ext in extensions:
                        ex = ext.strip()
                        for ff in f.walkfiles('*.%s' % ex):
                            buildfileFiles.append(ff)
            elif line[0] == '+':
                line = line[1:].strip()
                f = gp.joinpath(line[1:]) if line[0] in ('\\','/') else gp.joinpath(line)
                if f.isfile():
                    buildfileFiles.append(f)
                elif f.isdir():
                    for ext in extensions:
                        ex = ext.strip()
                        for ff in f.files('*.%s' % ex):
                            buildfileFiles.append(ff)
            elif line[0] == '-':
                line = line[1:].strip()
                f = gp.joinpath(line[1:]) if line[0] in ('\\','/') else gp.joinpath(line)
                if f.isfile():
                    exclude.append(f.normpath())
                elif f.isdir():
                    for ff in f.walkfiles():
                        exclude.append(ff.normpath())
    return strict, exclude, extensions, buildfileFiles

def buildfileParser0(lines, bf):
    print 'BUILDER: Parsing buildfile using parser version 0'
    exclude = False
    buildfileFiles = []
    strict = False
    extensions = ['py','txt','ini','cfg','res']
    firstline = lines[0] if len(lines) > 0 else ''
    secondline = lines[1] if len(lines) > 1 else ''
    if firstline.startswith('__extensions__'):
        extensions = firstline[14:].split(',')
        print 'BUILDER: Extensions: %s' % ','.join(extensions)
    else:
        lines.insert(0,'__extensions__ %s' % ','.join(extensions))
        bf.write_lines(lines)
    if firstline == '__strict__':
        print 'BUILDER: Strict build mode enabled'
        strict = True
    if secondline == '__strict__':
        print 'BUILDER: Strict build mode enabled'
        strict = True
    elif firstline == '__exclude__' or secondline == '__exclude__':
        exclude = []
        print 'BUILDER: Exclude mode enabled'
        for line in lines:
            if not line.startswith('__'):
                f = gp.joinpath(line[1:] if line[0] in ('/','\\') else line)
                if f.isfile():
                    exclude.append(f.normpath())
                elif f.isdir():
                    for ff in f.walkfiles():
                        exclude.append(ff.normpath())
    for line in lines:
        if not line.startswith('__'):
            f = gp.joinpath(line[1:] if line[0] in ('/','\\') else line)
            if f.isfile():
                buildfileFiles.append(f)
            elif f.isdir():
                for ff in f.walkfiles():
                    buildfileFiles.append(ff)
    return strict, exclude, extensions, buildfileFiles
 
def build():
    print ''
    if es.getargc() != 2:
        print 'BUILDER: Invalid syntax! Use: build <basename>'
        return
    ap = path.path(es.getAddonPath(es.getargv(1)))
    if not ap.isdir():
        print 'BUILDER: Couldn\'t find addon: %s' % es.getargv(1)
        return
    name = es.getargv(1)
    print 'BUILDER: Scanning main file for version info and downloadables'
    version = '1'
    main = ap.joinpath('%s.py' % name) if ap.joinpath('%s.py' % name).isfile() else ap.joinpath('es_%s.txt' % name)
    if not main.isfile():
        print 'BUILDER: Couldn\'t find main script file!'
        return
    uf = ap.joinpath('%s.upload' % name)
    L = []
    if uf.isfile():
        L = uf.lines(retain=False)
    downloadables = []
    done = []
    for line in main.lines(retain=False):
        if line.strip().startswith('info.version'):
            version = line.split('=')[1].strip().strip("'").strip('"')
            if len(L) != 0:
                x = 0
                for line in L:
                    if line.startswith('version'):
                        del L[x]
                        L.insert(x,'version %s' % version)
                        break
                    x += 1
            else:
                L.append('version %s' % version)
            print 'BUILDER: Version found: %s' % version
        if 'stringtable' in line and 'downloadables' in line:
            tmp = extractDownloadable(line)
            if tmp:
                downloadables.append(tmp)
        elif 'downloadable' in line:
            tmp = extractDownloadable(line)
            if tmp:
                downloadables.append(tmp)
    print 'BUILDER: Writing zipfile'
    bf = ap.joinpath('%s.build' % name)
    extensions = ['py','txt','ini','cfg','res']
    strict = False
    exclude = []
    if bf.isfile():
        print 'BUILDER: Buildfile found'
        bflines = bf.lines(retain=False)
        version1 = False
        for line in bflines:
            if line.startswith('parserversion 1'):
                version1 = True
                strict, exclude, extensions, buildfileFiles = buildfileParser1(bflines, bf)
                break
        if not version1:
            strict, exclude, extensions, buildfileFiles = buildfileParser0(bflines, bf)
    else:
        bf.write_text('__version__ 1\n__extensions__ %s' % ','.join(extensions))
    zf = ap.joinpath('%s_%s.zip' % (name,clean(version)))
    zip = zipfile.ZipFile(zf,'w')
    if not strict:
        for ext in extensions:
            ex = '.%s' % ext.strip()
            for f in ap.walkfiles('*%s' % ex):
                if f.normpath() not in exclude:
                    print 'BUILDER: Adding %s' % f.encode('ascii')
                    zip.write(f.encode('ascii'),gp.relpathto(f).encode('ascii'))
                    done.append(f)
        for downloadable in downloadables:
            if downloadable.normpath() not in exclude:
                print 'BUILDER: Adding %s' % downloadable.encode('ascii')
                zip.write(downloadable.encode('ascii'),gp.relpathto(downloadable).encode('ascii'))
                done.append(downloadable)
        cfg = gp.joinpath('cfg/%s.cfg' % name)
        if cfg.isfile():
            if cfg.normpath()  not in exclude:
                print 'BUILDER: Adding %s' % f.encode('ascii')
                zip.write(f.encode('ascii'),gp.relpathto(f).encode('ascii'))
                done.append(f)
    for f in buildfileFiles:
        if f not in done and f not in exclude:
            print 'BUILDER: Adding %s' % f.encode('ascii')
            zip.write(f.encode('ascii'),gp.relpathto(f).encode('ascii'))
            done.append(f)
    zip.close()
    uf.write_lines(L)
    print 'BUILDER: Done'
    print ''